//=============================================================================
// Frag.Ops  2003 Matt 'SquirrelZero' Farber
//=============================================================================
// Similar to ShadowProjector, just a lot better.  This shadow projector, while
// always in the same relative location to the player model, will dynamically
// adjust its FOV and also feed the ShadowBitmapMaterial new lighting data on
// the fly.  It uses a more exaggerated FOV than the standard UT200x shadows,
// because it looks so much cooler :).  It also uses the ShadowCrispness 
// setting of the ShadowController to set a larger ShadowBitmapMaterial size.
// Note that the bigger the shadow material the more data is being passed to
// video each frame, so performance will worsen exponentially as you go higher.
// I had to do a lot of little tweaks and hack some settings in to get the
// projector to do what i wanted, and not cut off at strange angles.
//=============================================================================
class Effect_ShadowProjector extends Projector;

var() vector LightDirection;
var() float LightDistance, InterpolationRate, MaxFOV, FadeSpeed, DarknessScale;
var ShadowBitmapMaterial ShadowTexture;
var bool bFadeIn;

function PostBeginPlay()
{
	Super(Actor).PostBeginPlay();
}

// these don't need to tick, we update all shadows at the same time in the controller
function Tick(float dt) {}

// this turns the shadow off
function DisableShadow()
{
	// detach
	DetachProjector();

	// stop shadow texture from being reuploaded to video here	
	if (ShadowTexture != None)
	{
		ShadowTexture.Dirty = false;
		ShadowTexture.ShadowActor = None;
	}
}

// initialize
function InitializeFor(Effect_ShadowController ShadowController)
{
	if (ShadowController.Owner != None)
	{
		// set the Owner
		SetOwner(ShadowController.Owner);
		
		// allocate the shadow texture
		switch (ShadowController.ShadowCrispness)
		{
			case Medium:
				ShadowTexture = ShadowBitmapMaterial(Level.ObjectPool.AllocateObject(class'Effect_ShadowBitmapMaterialMedium'));
				break;
		
			case High:
				ShadowTexture = ShadowBitmapMaterial(Level.ObjectPool.AllocateObject(class'Effect_ShadowBitmapMaterialHigh'));
				break;
			
			case Maximum:
				ShadowTexture = ShadowBitmapMaterial(Level.ObjectPool.AllocateObject(class'Effect_ShadowBitmapMaterialMax'));
				break;
			
			Default:
				ShadowTexture = ShadowBitmapMaterial(Level.ObjectPool.AllocateObject(class'Effect_ShadowBitmapMaterialLow'));
				break;

		}

		// set projector texture
		ProjTexture = ShadowTexture;
		
		// initialize the shadow texture
		if (ShadowTexture != None)
		{
			ShadowTexture.Invalid = false;
			ShadowTexture.ShadowActor = Owner;
			ShadowTexture.bBlobShadow = false;
			ShadowTexture.CullDistance = CullDistance; 
		}
		else
		{
			log(Name$".InitializeFor: No shadow texture!  Escaping...");
			Destroy();
		}
	}
	else
		log(Name$".InitializeFor: No Owner!");
}

// updates this shadow's location, rotation, and FOV
function UpdateShadow(float dt, int LN, Effect_ShadowController ShadowController)
{
	local Plane BoundingSphere;
	local Actor ShadowLight;
	local vector Diff, ShadowLocation, Origin;
	local rotator LightRotation, AdjustedRotation;
	local float Interpolation;
	local bool bFadeOut;

	// detach projector
	DetachProjector(true);

	// fallback, don't draw if hidden or no shadow texture
	if (Owner == None || Owner.bHidden || ShadowTexture == None)
		return;

	// fallback and destroy
	if (ShadowTexture.Invalid)
	{
		Destroy();
		return;
	}

	// cull more if we haven't rendered this pawn in 5 seconds
	if (Level.TimeSeconds - Owner.LastRenderTime > 5)
		CullDistance = 0.5*Default.CullDistance;
	else
		CullDistance = Default.CullDistance;

	// cull shadows much earlier if below min framerate, important
	if (Level.bDropDetail)
		ShadowTexture.CullDistance = 0.6*CullDistance;
	else
		ShadowTexture.CullDistance = CullDistance;

	// in case shadow was disabled earlier
	ShadowTexture.ShadowActor = Owner;

	// set light
	ShadowLight = ShadowController.Lights[LN].CurrentLight;

	// fade out if necessary
	if (ShadowController.Lights[LN].LastLight != None)
	{
		if (ShadowTexture.ShadowDarkness > 5)
		{
			ShadowLight = ShadowController.Lights[LN].LastLight;
			bFadeOut = true;
		}
		else
		{
			ShadowController.Lights[LN].LastLight = None;
			bFadeOut = false;
			bFadeIn = true;
			DarknessScale = 0;
		}
	}

	// fallback if no more lights after fadeout
	if (ShadowLight == None)
		return;

	// get the direction of the light
	Diff = ShadowLight.Location - Owner.Location;

	// set light distance
	if (ShadowLight.LightEffect == LE_Sunlight)
		LightDistance = ShadowController.MaxLightDistance*0.3;
	else
		LightDistance = FClamp(VSize(Diff), ShadowController.MaxLightDistance*0.1, ShadowController.MaxLightDistance*0.3);
	
	// get a location along the path of the light slightly away from center of player
	ShadowLocation = Owner.Location + 4*Normal(Diff);
	if (ShadowLocation.Z < Owner.Location.Z)
		ShadowLocation.Z = Owner.Location.Z+4;

	// set location
	SetLocation(ShadowLocation + vect(0,0,-8));

	// determine correct rotation, interpolate
	Origin = ShadowLocation + ShadowTexture.LightDirection * ShadowTexture.LightDistance;
	Interpolation = FMin(1.0, (dt*InterpolationRate));
	Origin += (ShadowLight.Location - Origin) * Interpolation;	
	Diff = ShadowLocation - Origin;
	LightRotation = rotator(Diff);

	// calculate FOV
	BoundingSphere = Owner.GetRenderBoundingSphere();
	FOV = (Atan(BoundingSphere.W*2 + 160, LightDistance) * 180/PI);

	// set rotation, compensate for FOV warping -- kinda hackish, but fixes shadows that 
	// bend so much they detach from the pawn
	AdjustedRotation = rotator(Owner.Location-ShadowLight.Location);
	AdjustedRotation.Pitch = Clamp(AdjustedRotation.Pitch, -20500, -9500);
	SetRotation(AdjustedRotation);

	// determine correct direction of light
	LightDirection = -vector(AdjustedRotation)*LightDistance;
	
	// set light direction
	ShadowTexture.LightDirection = Normal(LightDirection);

	// set lightdistance
	ShadowTexture.LightDistance = LightDistance;

	// update shadow texture FOV
	ShadowTexture.LightFOV = FOV;
	
	// set the drawscale, exaggerate a bit
	SetDrawScale((LightDistance*0.82) * tan(0.5*FOV*PI/180) / (0.45*ShadowTexture.USize));

	// fade out gracefully
	if (bFadeOut)
		ShadowTexture.ShadowDarkness = Max(ShadowTexture.ShadowDarkness - (FadeSpeed*dt), 0);
	else
	{
		ShadowTexture.ShadowDarkness = 140*(1.0-(FClamp(VSize(ShadowLight.Location-Owner.Location)/ShadowController.MaxLightDistance, 0.0, 1.0))) + 70;
		if (bFadeIn && DarknessScale < 1.0)
		{
			DarknessScale = FMin(DarknessScale + ((FadeSpeed*0.007)*dt), 1.0);
			ShadowTexture.ShadowDarkness = float(ShadowTexture.ShadowDarkness)*DarknessScale;
		}
		else
			bFadeIn = false;
	}

	// dirty texture for reuploading to video
	ShadowTexture.Dirty = true;

	// reattach projector
	AttachProjector();
}

function Destroyed()
{
	// free shadow texture from the object pool
	if (ShadowTexture != None)
	{
		ShadowTexture.ShadowActor = None;
		
		if (!ShadowTexture.Invalid)
			Level.ObjectPool.FreeObject(ShadowTexture);

		// must set to none
		ShadowTexture = None;
		ProjTexture = None;
	}
	Super.Destroyed();
}

defaultproperties
{
	// this lets us have shadows that cast onto other players
	bProjectActor=true
	// ... made possible by this, so it doesn't project onto the pawn owning it
	bNoProjectOnOwner=true
	// how quickly we should interpolate
	InterpolationRate=2.5
	// this should be dynamically updated also
	MaxTraceDistance=165
	// maximum FOV
	MaxFOV=80.0
	// how quickly to fade in/out
	FadeSpeed=500.0

	bProjectOnParallelBSP=true
	bProjectOnAlpha=false
	bClipBSP=true
	bGradient=false
	bStatic=false
	bOwnerNoSee=true
	bDynamicAttach=true
	RemoteRole=ROLE_None
	CullDistance=1200.0
}