//=============================================================================
// Frag.Ops  2003 Matt 'SquirrelZero' Farber
//=============================================================================
// Ambient map creatures.  Models by Psionic of Psionic3D, thanks bud :).
//=============================================================================
class FOAmbientCreatures extends Actor
	abstract;

#exec OBJ LOAD FILE=FOCreatureSkins.utx

// type of movement path this creature follows
var() enum CMovementType
{
	MT_StraightLine,
	MT_Circular,
	MT_Random
}MovementType;

// seed that determines how many times creature will perform
// each animation before changing.  2 is a good IdleLoopSeed,
// 6-8 are good WalkLoopSeed's.  Setting IdleLoopSeed to 0 makes
// the creature constantly moving.
var() int IdleLoopSeed, WalkLoopSeed;

// maximum distance creature will travel before turning 
var() float MaxDistanceBeforeTurn;

// scale of the circle when MT_Circular
var() float CircleSize;

// creature anims
var() name IdleAnim, TurnAnim, WalkAnim, RunAnim;
var() float IdleAnimRate, TurnAnimRate, WalkAnimRate, RunAnimRate;

// speed creature walks and runs at (anims do not scale automatically)
var() float WalkSpeed, RunSpeed;

// starting position and rotation of creature, location when turned
var vector StartLocation, TurnedLocation;
var rotator StartRotation, TurnedRotation;

// number of times to loop the idle anim at state runtime
var int NumLoops, NumWalkLoops, LoopCount, BodyLength;

// does this creature run?
var bool bCanRun;

simulated function PostBeginPlay()
{
	Super.PostBeginPlay();

	// initialize startlocation/rotations
	StartLocation = Location;
	TurnedLocation = StartLocation;
	StartRotation = Rotation;

	// initialise desiredrotation
	DesiredRotation = Rotation;

	if (Level.NetMode == NM_DedicatedServer)
		GotoState('');
}

//=============================================================================
// creature states, controls walking, running, turning, and idle

simulated function Reset()
{
	if (Level.NetMode == NM_DedicatedServer)
		return;
	
	GotoState('');
	SetLocation(StartLocation);
	TurnedLocation = StartLocation;
	SetRotation(StartRotation);
	DesiredRotation = Rotation;
	StopAnimating();
	Velocity = vect(0,0,0);
	SetTimer(0.0, false);
	NumLoops = 0;
	NumWalkLoops = 0; 
	LoopCount = 0;
	GotoState(InitialState);
}

simulated state TimedStartup
{
	simulated function BeginState()
	{
		SetTimer(3.0, false);
	}

	simulated function Timer()
	{
		SetPhysics(PHYS_Projectile);
		GotoState('Idle');
	}
}

simulated state Idle
{
	simulated function BeginState()
	{
		// random idle repetitions
		if (NumLoops >= IdleLoopSeed)
			NumLoops = 1;
		else
			NumLoops++;

		// reset loop count
		LoopCount = 0;

		// blend anims
		AnimBlendToAlpha(0, 0.5, 0.2);

		// loop idle anim
		LoopAnim(IdleAnim,IdleAnimRate);
	}


	// when the animation ends, check if we've looped enough times,
	// then move to the walking state.  Otherwise do nothing.
	simulated event AnimEnd(int Channel)
	{
		LoopCount++;
		if (LoopCount >= NumLoops)
		{
			switch(MovementType)
			{			
				case MT_Circular:
					GotoState('CircularWalking');
					break;

				default:
					GotoState('Walking');
					break;
			}
		}
	}

	// slow creature to a halt
	simulated function Tick(float dt)
	{
		if (VSize(Velocity) < 5 && Velocity != vect(0,0,0))
			Velocity = vect(0,0,0);
		else
			Velocity *= 0.6;
	}

	simulated function EndState()
	{
	}
}

simulated state StraightTurn
{
	simulated function BeginState()
	{
		// set location last turned
		TurnedLocation = Location;
		TurnedRotation = rot(0,0,0);

		// turn in the opposite direction along a straightline
		DesiredRotation.Yaw = -DesiredRotation.Yaw;
		bRotateToDesired = true;

		// blend anims
		AnimBlendToAlpha(0, 0.5, 0.2);

		// loop idle anim
		LoopAnim(TurnAnim, TurnAnimRate);

		// initialize timed rotation check 
		SetTimer(0.14,false);
	}
	
	// slow creature to a halt
	simulated function Tick(float dt)
	{
		if (VSize(Velocity) < 5 && Velocity != vect(0,0,0))
			Velocity = vect(0,0,0);
		else
			Velocity *= 0.6;
	}

	// check if completed turn
	simulated function Timer()
	{
		if (Rotation.Yaw == TurnedRotation.Yaw)
		{
			GotoState('Walking');
			return;
		}
		TurnedRotation.Yaw = Rotation.Yaw;
		SetTimer(0.14,false);
	}

	simulated function EndState()
	{
		// stop timer loop
		SetTimer(0,false);
	}
}

simulated state RandomTurn extends StraightTurn
{
	simulated function BeginState()
	{
		local rotator RotationCheck;

		// set location last turned
		TurnedLocation = Location;
		TurnedRotation = rot(0,0,0);

		// first, check if we're too far away from the start point
		if (VSize(Location-StartLocation) >= MaxDistanceBeforeTurn)
		{
			RotationCheck = rotator(StartLocation-Location);
			DesiredRotation.Yaw = RotationCheck.Yaw;
		}
		else // then check to the left
		{

			RotationCheck = Rotation;
			RotationCheck.Yaw -= 16384;
			if (!FastTrace((Location+vect(0,0,5)) + (BodyLength*vector(RotationCheck)), Location+vect(0,0,5)))
				DesiredRotation.Yaw += 20000+(FRand()*13000);
			else // check to the right
			{
				RotationCheck.Yaw += 32767;
				if (!FastTrace((Location+vect(0,0,5)) + (BodyLength*vector(RotationCheck)), Location+vect(0,0,5)))
					DesiredRotation.Yaw += -20000+(FRand()*-13000);
				else // turn more softly
					DesiredRotation.Yaw += 10000+(FRand()*23000);
			}
		}
		bRotateToDesired = true;

		// blend anims
		AnimBlendToAlpha(0, 0.5, 0.2);

		// loop idle anim
		LoopAnim(TurnAnim, TurnAnimRate);

		// initialize timed rotation check 
		SetTimer(0.14,false);
	}
}

simulated state Walking
{
	simulated function BeginState()
	{
		// random idle repetitions
		if (NumWalkLoops >= WalkLoopSeed)
			NumWalkLoops = WalkLoopSeed/2;
		else
			NumWalkLoops++;

		// reset loop count
		LoopCount = 0;

		// blend anims
		AnimBlendToAlpha(0, 0.5, 0.2);

		// loop walking anim
		LoopAnim(WalkAnim,WalkAnimRate);

		// initialize timed distance check 
		SetTimer(0.14,false);	
	}

	// wind up to full speed
	simulated function Tick(float dt)
	{
		// is creature currently slower than top speed?
		if (VSize(Velocity) <= WalkSpeed)
			Velocity += vector(Rotation) * (WalkSpeed/10);

		// if faster than top speed, enforce maximum walkspeed
		if (VSize(Velocity) > WalkSpeed)
			Velocity = WalkSpeed * vector(Rotation);

		// for random motion, treat walking like a bump 'n go
		if (MovementType == MT_Random || bCollideActors)
		{
			if (!FastTrace((Location+vect(0,0,5)) + (BodyLength*vector(Rotation)), Location+vect(0,0,5)))
			{
				if (MovementType == MT_Random)
					GotoState('RandomTurn');
				else
					GotoState('StraightTurn');
			}
		}
	}

	// increment anim loop counts
	simulated event AnimEnd(int Channel)
	{
		LoopCount++;
	}

	// check for state change 
	simulated function Timer()
	{
		local float DistanceTraveled;

		// get distance traveled since last turn 
		DistanceTraveled = VSize(TurnedLocation-Location);

		if (LoopCount >= NumWalkLoops || DistanceTraveled >= MaxDistanceBeforeTurn)
		{
			if (DistanceTraveled >= MaxDistanceBeforeTurn)
			{	
				RotationRate.Yaw = -RotationRate.Yaw;
				switch(MovementType)
				{
					case MT_StraightLine:
						// reverse rotation direction
						GotoState('StraightTurn');
						break;

					case MT_Random:
						GotoState('RandomTurn');
						break;
				}
			}
			else
				GotoState('Idle');

			return;
		}
		SetTimer(0.14, false);
	}

	simulated function EndState()
	{
	}
}

simulated state CircularWalking extends Walking
{
	simulated function Tick(float dt)
	{
		// is creature currently slower than top speed?
		if (VSize(Velocity) <= WalkSpeed)
			Velocity += vector(Rotation) * (WalkSpeed/10);

		// if faster than top speed, enforce maximum walkspeed
		if (VSize(Velocity) > WalkSpeed)
			Velocity = WalkSpeed * vector(Rotation);

		// constantly turn 
		DesiredRotation.Yaw += CircleSize;
		if (DesiredRotation.Yaw >= 65535)
			DesiredRotation.Yaw = 0;

		bRotateToDesired = true;
	}

	// when the animation ends, check if we've looped enough times,
	// then move to the walking state.  Otherwise do nothing.
	simulated event AnimEnd(int Channel)
	{
		LoopCount++;
		if (LoopCount >= NumWalkLoops && IdleLoopSeed > 0)
		{
			GotoState('Idle');
		}
	}

	simulated function Timer() {}
}

simulated state Running extends Walking
{
	simulated function BeginState()
	{}

	simulated function EndState()
	{}
}

defaultproperties
{
	// movement
	MovementType=MT_StraightLine
	WalkSpeed=80.0
	RunSpeed=120.0

	// settings
	IdleLoopSeed=2
	WalkLoopSeed=7
	CircleSize=100
	MaxDistanceBeforeTurn=570
	BodyLength=30

	// anims
	IdleAnim=Idle
	IdleAnimRate=0.6	
	WalkAnim=Walk
	WalkAnimRate=0.7
	RunAnim=Walk
	RunAnimRate=1.3
	TurnAnim=Walk
	TurnAnimRate=0.5

	DrawType=DT_Mesh
	DrawScale=0.33
	Physics=PHYS_None
	InitialState=TimedStartup
	RemoteRole=ROLE_SimulatedProxy
	RotationRate=(Yaw=-26000)
	AmbientGlow=40
	bFixedRotationDir=True
	bAlwaysRelevant=true
	bNoDelete=true
	bCanRun=false
	bRotateToDesired=true
	bWorldGeometry=false
	bCollideWorld=false
	bCollideActors=false
	bCollideWhenPlacing=false
	bBlockActors=false
	bBlockPlayers=false
	bDirectional=true
	bShouldBaseAtStartup=true
	bOrientOnSlope=true
	bSkipActorPropertyReplication=true
	bUseDynamicLights=true
}