//=============================================================================
// Frag.Ops  2003 Matt 'SquirrelZero' Farber
//=============================================================================
// FragOps_ChainableObjective.uc
// the base objective class, the only one not included in this is the escape
// volume, which can be found under volumes in your editor.
//=============================================================================
class FragOps_ChainableObjective extends Actor
	notplaceable;

// team which will be completing this objective
// This also determines which chain it's placed into.
// 0 = Mercenaries
// 1 = Special ops
var() byte ObjectiveTeam;

// where this objective stands in the chain, or its "link".
// 0 = first link, 1 = second, etc.  If you only have one, 
// leave it at 0.  Each team receives its own seperate chain.  
// Note that you can make several different objectives the same 
// link number, and it will only require players to complete 
// one of them to move onto the next link.  For example, a 
// room with 4 doors, but only one needs to be opened to complete 
// the objective.  For this you would make all 4 doors the same
// LinkNumber.
var() byte LinkNumber;

// optional event to trigger when objective is initially triggered
var() name StartedEvent;

// optional message that will broadcast to all players when someone has begun
// completing this objective, ignored if not a timed objective.
var() string StartedMessage;

// optional message to broadcast when a team fails to complete this
// objective (i.e. time runs out).  In InteractObjectives, this will also
// broadcast when a player fails to complete the objective within the
// specified TriggerTime.  The only time the failed message
// is read from non-InteractObjectives is if a couple requirements
// are met:
// -- a) The other team has _no objectives_ in their chain
// -- b) This is the last objective in the chain
var() string FailedMessage;

// optional event to trigger when objective is completed
var() name CompletedEvent;

// optional message that will broadcast to all players when this objective
// has been completed
var() string CompletedMessage;

// optional sound to play when this objective is completed
var() sound CompletedSound;

// time after objective is completed before either allowing the next
// objective or ending the round.  This is useful for creating cinematic
// sequences where you want the game to "wait" for things to happen before
// moving on to the next link.  Optional.
var() float SequenceTime;

// optional credit bonus to award the player who completes this objective
var() int CreditBonus;

// short description of this objective, used in the briefing screen, very
// important!  32 characters MAX!
var() string ShortDescription;

// completing this objective is optional
var() bool bCompletionOptional;

// don't award any points for completing this objective
var() bool bNoPointsForCompletion;

var bool bCompleted, bIsIdle, bLinkComplete, bFinishedObjective;

replication
{
	// things to send to the clients
	reliable if ((bNetDirty) && (Role == ROLE_Authority))
		bLinkComplete;
}

function Cancel();

// sets this objective up
function PostBeginPlay()
{
	if (FragOpsMission(Level.Game) == None)
		return;
	
	// add ourselves to the objective chain on map start
	if (ObjectiveTeam == 0)
		FragOpsMission(Level.Game).MC_ObjectiveChain[LinkNumber] = self;
	else
		FragOpsMission(Level.Game).SO_ObjectiveChain[LinkNumber] = self;
}

// resets to the initial settings
function Reset()
{
	bCompleted = false;
	bLinkComplete = false;
	bFinishedObjective = false;

	GotoState(InitialState);
}

// for timed objectives
function StartObjective(Pawn Other)
{
	// trigger the start event
	TriggerEvent(StartedEvent, self, Other);
}

// called when this objective is triggered
function OpenObjective(Pawn Other)
{
	if (FOPawn(Other) == None || Other.PlayerReplicationInfo == None || Other.PlayerReplicationInfo.Team.TeamIndex != ObjectiveTeam || bFinishedObjective || FOPawn(Other).bRestrictDamage || FragOpsGame(Level.Game).bRoundEnded)
		return;

	if (ObjectiveTeam == 0)
	{
		if ((bLinkComplete || !bCompleted) && ((FragOpsMission(Level.Game).MC_ObjectiveChain.Length <= 1) || LinkNumber == 0 || (FragOpsMission(Level.Game).MC_ObjectiveChain[LinkNumber-1].bCompleted || FragOpsMission(Level.Game).MC_ObjectiveChain[LinkNumber-1].bCompletionOptional) || (FragOpsMission(Level.Game).MC_ObjectiveChain[LinkNumber-1].bLinkComplete)) && (!IsInstate('CompletedObjective')))
			StartObjective(Other);
	}
	else if (ObjectiveTeam == 1)
	{
		if ((bLinkComplete || !bCompleted) && ((FragOpsMission(Level.Game).SO_ObjectiveChain.Length <= 1) || LinkNumber == 0 || (FragOpsMission(Level.Game).SO_ObjectiveChain[LinkNumber-1].bCompleted || FragOpsMission(Level.Game).SO_ObjectiveChain[LinkNumber-1].bCompletionOptional) || (FragOpsMission(Level.Game).SO_ObjectiveChain[LinkNumber-1].bLinkComplete)) && (!IsInstate('CompletedObjective')))
			StartObjective(Other);
	}
}

// the initial state for this objective.  Basically the "idle" state
state WaitingObjective
{
	function Cancel()
	{}

	function BeginState()
	{
		SetCollision(true,false,false);
	}
}

// the "completed" state for this objective.  After completion the
// objective rests in this state so it cannot be triggered anymore.
state CompletedObjective
{
	function OpenObjective(Pawn Other)
	{}

	function Cancel()
	{}

	function StartObjective(Pawn Other)
	{}

	function EndState()
	{
		bIsIdle = false;
	}

	function BeginState()
	{
		if (CompletedSound != None)
			PlaySound(CompletedSound,,TransientSoundVolume*1.3);
		bIsIdle = true;
		if (!bLinkComplete && CompletedMessage != "")
			FragOpsMission(Level.Game).BroadcastLocalizedMessage(class'Interface_ObjectiveMessage',2,None,None,self);
		if (!bLinkComplete)
			UpdateThisLink();
		bLinkComplete = true;
		SetCollision(false,false,false);
		// switch 0 = open message, 1 = failed message, 2 = completed message, 5 = used internally by gamemessage
	}
Begin:
	if (CheckLinks())
		RestrictPlayers();
	Sleep(SequenceTime);
	if (ObjectiveTeam == 0 && FragOpsMission(Level.Game).MC_ObjectiveChain[LinkNumber] != self)
	{
		FragOpsMission(Level.Game).MC_ObjectiveChain[LinkNumber].bCompleted = true;
	}
	else if (ObjectiveTeam == 1 && FragOpsMission(Level.Game).SO_ObjectiveChain[LinkNumber] != self)
	{
		FragOpsMission(Level.Game).SO_ObjectiveChain[LinkNumber].bCompleted = true;
	}
	bCompleted = true;
	FragOpsMission(Level.Game).UpdateChain(self);	
	if (CheckLinks())
	{
		FragOpsMission(Level.Game).BroadcastLocalizedMessage(class'Interface_GameMessage',5,None,None,self);
	}
}

function UpdateThisLink()
{
	local FragOps_ChainableObjective FCO;

	foreach AllActors(class'FragOps_ChainableObjective',FCO,'') 
	{
		if (FCO.LinkNumber == LinkNumber && FCO != self && FCO.ObjectiveTeam == ObjectiveTeam)
		{
			FCO.bLinkComplete = true;
		}
	}
}

// usually called from outside classes when a player completes this objective
function Completed(Pawn Other)
{
	if (FragOpsGame(Level.Game).bRoundEnded || bFinishedObjective)
		return;

	bFinishedObjective = true;

	if (((ObjectiveTeam == 0 && !FragOpsMission(Level.Game).MC_ObjectiveChain[LinkNumber].bLinkComplete) || (ObjectiveTeam == 1 && !FragOpsMission(Level.Game).SO_ObjectiveChain[LinkNumber].bLinkComplete)) && !bNoPointsForCompletion && Other != None)
	{
		if (ObjectiveTeam == 1)
		{
			if ((FragOpsMission(Level.Game).SO_ObjectiveChain.Length-1) == LinkNumber)
			{
				Other.PlayerReplicationInfo.Score += 2;
				FragOps_PRI(Other.PlayerReplicationInfo).DamageDone += 200;
			}
			else
			{
				Other.PlayerReplicationInfo.Score += 1;
				FragOps_PRI(Other.PlayerReplicationInfo).DamageDone += 100;
			}
		}
		else
		{
			if ((FragOpsMission(Level.Game).MC_ObjectiveChain.Length-1) == LinkNumber)
			{
				Other.PlayerReplicationInfo.Score += 2;
				FragOps_PRI(Other.PlayerReplicationInfo).DamageDone += 200;
			}
			else
			{
				Other.PlayerReplicationInfo.Score += 1;
				FragOps_PRI(Other.PlayerReplicationInfo).DamageDone += 100;
			}
		}
	}
	if (SequenceTime > 0)
		TriggerEvent(CompletedEvent, self, Other);
	GotoState('CompletedObjective','Begin');
	if (SequenceTime <= 0)
		TriggerEvent(CompletedEvent, self, Other);
}

// check if this is the last link in the chain to be completed
function bool CheckLinks()
{
	if (ObjectiveTeam == 0)
	{
		if (LinkNumber == (FragOpsMission(Level.Game).MC_ObjectiveChain.Length-1))
			return true;
	}
	else if (ObjectiveTeam == 1)
	{
		if (LinkNumber == (FragOpsMission(Level.Game).SO_ObjectiveChain.Length-1))
			return true;
	}
	return false;
}

// stop players from taking damage if this objective ends the round
function RestrictPlayers()
{
	local controller C;

	for (C=Level.ControllerList; C!=None; C=C.nextController )
	{
		if (C.Pawn != None && C.Pawn.Health > 0)
		{
			FOPawn(C.Pawn).bRestrictDamage = true;
		}
	}
}

event Trigger(actor Other, pawn EventInstigator)
{
	bFinishedObjective = true;
}

defaultproperties
{
	Texture=Texture'FOEditorTextures.Objectives.ObjectiveMain'
	bHidden=true
	bHiddenEd=false
	bEdShouldSnap=true
	ObjectiveTeam=0
	LinkNumber=0
	bBlockPlayers=false
	bBlockActors=false
	bCollideActors=true
	bCollideWorld=false
	bCollideWhenPlacing=false
	SequenceTime=1.000
	InitialState=WaitingObjective
	bAlwaysRelevant=true
	bCompleted=false
	FailedMessage="Mercenaries failed their objective!"
	CompletedMessage="Mercenaries completed their objective!"
	CompletedSound=Sound'FOWeaponSounds.Objectives.PanelComplete'
	bNoPointsForCompletion=false
	CollisionRadius=15.000000
	CollisionHeight=15.000000
	RemoteRole=ROLE_DumbProxy
	bStatic=false
	bNoDelete=true
}