//=============================================================================
// Frag.Ops  2003 Matt 'SquirrelZero' Farber
//=============================================================================
// Obj_DestroyableObjective.uc
// An objective that can be destroyed with gunfire and other damage methods.
// StartedMessage is broadcast when the objective has reached 1/2 damage.
//=============================================================================
class Obj_DestroyableObjective extends FragOps_ChainableObjective
	placeable;

// Karma Object type
enum EKarmaType
{
	KLead,
	KSteel,
	KRubber,
	KWood,
	KFeather
};

// ChunkKarma, can use karma for chunks instead
struct ChunkKarmaProps
{
	var() bool bUseKarmaInstead;
	var() EKarmaType KarmaType;
};

// Chunkrecord
struct ChunkRecord
{
	var() StaticMesh ChunkMesh;
	var() editinline float ChunkScale;
	var() Sound ChunkBounceSound;
	var() float ChunkVelocity;
	var() float ChunkLifeSpan;
	var() editinline ChunkKarmaProps ChunkKarma;
};

// these chunks will fly out everywhere when objective is destroyed
var() editinline array<ChunkRecord> ExplodedChunks;

// optional effects to spawn when objective is destroyed, keep these
// clean and fast, since we are already spawning a LAW explosion
// from the missile.
var() editinline class<Emitter> ExplodedEmitter;
var() editinline class<xEmitter> ExplodedXEmitter;

// mesh to set to when destroyed
var() StaticMesh DestroyedMesh;

// maximum damage this objective can withstand before destruction
var() int MaxDamage;

// location offset to spawn emitter/chunks
var() vector EffectSpawnOffset;

var int DamageTaken;
var bool bHasBroadcast;
var StaticMesh OrigMesh;

function PostBeginPlay()
{
	Super.PostBeginPlay();
	OrigMesh = StaticMesh;
}

function bool bOpenObjective(Pawn Other)
{
	if (FOPawn(Other) == None || Other.PlayerReplicationInfo == None || Other.PlayerReplicationInfo.Team.TeamIndex != ObjectiveTeam || bFinishedObjective || FOPawn(Other).bRestrictDamage || FragOpsGame(Level.Game).bRoundEnded)
		return false;

	if (ObjectiveTeam == 0)
	{
		if ((bLinkComplete || !bCompleted) && ((FragOpsMission(Level.Game).MC_ObjectiveChain.Length <= 1) || LinkNumber == 0 || (FragOpsMission(Level.Game).MC_ObjectiveChain[LinkNumber-1].bCompleted || FragOpsMission(Level.Game).MC_ObjectiveChain[LinkNumber-1].bCompletionOptional) || (FragOpsMission(Level.Game).MC_ObjectiveChain[LinkNumber-1].bLinkComplete)) && (!IsInstate('CompletedObjective')))
			return true;
	}
	else if (ObjectiveTeam == 1)
	{
		if ((bLinkComplete || !bCompleted) && ((FragOpsMission(Level.Game).SO_ObjectiveChain.Length <= 1) || LinkNumber == 0 || (FragOpsMission(Level.Game).SO_ObjectiveChain[LinkNumber-1].bCompleted || FragOpsMission(Level.Game).SO_ObjectiveChain[LinkNumber-1].bCompletionOptional) || (FragOpsMission(Level.Game).SO_ObjectiveChain[LinkNumber-1].bLinkComplete)) && (!IsInstate('CompletedObjective')))
			return true;
	}
}

function TakeDamage(int Damage, Pawn instigatedBy, Vector HitLocation, Vector Momentum, class<DamageType> damageType)
{
	local vector start, newstart;
	local rotator rot;
	local int i;
	local Effect_LAWChunk NewChunk;
	local Effect_LAWEmitterSpawner LAWEmitterSpawner;

	if (InstigatedBy == None || !bOpenObjective(InstigatedBy))
		return;

	if (DamageType.Name == 'DamType_Machete' || DamageType.Name == 'DamType_AKBayonet' || DamageType.Name == 'DamType_MachetePoison')
		DamageTaken += Damage/3;
	else
		DamageTaken += Damage;

	if (DamageTaken >= (MaxDamage/2) && !bHasBroadcast && DamageTaken < MaxDamage && StartedMessage != "")
	{
		FragOpsMission(Level.Game).BroadcastLocalizedMessage(class'Interface_ObjectiveMessage',0,None,None,self);
		bHasBroadcast = true;
	}

	if (DamageTaken < MaxDamage)
		return;

	SetStaticMesh(None);
	SetCollision(false,false,false);
	
	// spawn the explosion emitter
	if (ExplodedEmitter != None || ExplodedXEmitter != None)
	{
		LAWEmitterSpawner = Spawn(class'Effect_LAWEmitterSpawner',,, (Location + EffectSpawnOffset), rotator(vect(0,0,1)));
		LAWEmitterSpawner.EmitterClass = ExplodedEmitter;
		LAWEmitterSpawner.XEmitterClass = ExplodedXEmitter;
		LAWEmitterSpawner.DoSpawn();
	}

	// complete
	Completed(instigatedBy);

	if (DestroyedMesh != None)
	{
		NewChunk = Spawn(class'Effect_LAWChunk',,, Location, Rotation);
		if (NewChunk != None)
		{
			NewChunk.SetStaticMesh(DestroyedMesh);
			NewChunk.SetDrawScale(DrawScale);
		}
		return;
	}

	// spawn some chunks
	start = Location + EffectSpawnOffset;
	for (i=0;i<ExplodedChunks.Length;i++)
	{
		rot = rotator(vect(0,0,1));
		rot.yaw += FRand()*32000-16000;
		rot.pitch += FRand()*32000-16000;
		rot.roll += FRand()*32000-16000;
		NewStart.X = (Start.X - 2) + (FRand()*4);
		NewStart.Y = (Start.Y - 2) + (FRand()*4);
		NewChunk = Spawn(class'Effect_LAWChunk',,, Start, rot);
		if (ExplodedChunks[i].ChunkMesh != None && NewChunk != None)
		{
			NewChunk.SetStaticMesh(ExplodedChunks[i].ChunkMesh);
			NewChunk.SetDrawScale(ExplodedChunks[i].ChunkScale);
			NewChunk.ImpactSound = ExplodedChunks[i].ChunkBounceSound;
			NewChunk.NewLifeSpan = ExplodedChunks[i].ChunkLifeSpan;
			if (ExplodedChunks[i].ChunkVelocity >= 1.0)
				NewChunk.Speed = ExplodedChunks[i].ChunkVelocity;
			NewChunk.SetKarma(ExplodedChunks[i].ChunkKarma.bUseKarmaInstead,int(ExplodedChunks[i].ChunkKarma.KarmaType));
		}
	}
}

function Reset()
{
	Super.Reset();
	SetStaticMesh(OrigMesh);
	SetCollision(true,true,true);
	DamageTaken = 0;
	bHasBroadcast = false;
}

// the initial state for this objective.  Basically the "idle" state
state WaitingObjective
{
	function Cancel()
	{}

	function BeginState()
	{}
}

defaultproperties
{
	Physics=PHYS_None
	bHidden=false
	bWorldGeometry=true
	bCollideWorld=false
	bCollideActors=true
	bBlockPlayers=true
	bBlockActors=true
	bUseDynamicLights=true
	bCollideWhenPlacing=false
	bCanBeDamaged=true
	bProjTarget=true
	bAcceptsProjectors=true
	bAlwaysRelevant=true
	CompletedSound=None
	DrawType=DT_StaticMesh
	StaticMesh=StaticMesh'V-Meshes3.FuelTankDual'
	DestroyedMesh=None
	MaxDamage=800
	StartedMessage="Fuel Tanks are damaged!"
	CollisionHeight=+000001.000000
	CollisionRadius=+000001.000000
}