//=============================================================================
// Frag.Ops  2003 Matt 'SquirrelZero' Farber
//=============================================================================
// FragOps_UsedObjective.uc
// the base objective class for all objectives that require players to press
// the "Interact" button to trigger it.  These can either be timed, to make the
// player "use" them for a period of time, or to make them instantly triggered
// you can set TriggerTime to 0.0
//=============================================================================
class Obj_InteractObjective extends FragOps_ChainableObjective
	placeable
	hidecategories(Lighting,LightColor,Karma,Force,Wind);

// time it takes between when this objective is initially
// triggered and when it completes.  If this is a "used"
// objective, the person cannot move while they are 
// attempting to complete it.  0.0 = flag this objective
// as complete as soon as it's triggered.
// Note -- if this is the final link in the chain, players won't take damage
// after the TriggerTime is expired even if the SequenceTime is greater than 0.0,
// but there is a reason for this.  The objective is officially flagged as "complete"
// after the TriggerTime expires, so if it is the final objective, then the team
// has won the round and therefore no more players may be killed.  Your sequence
// will still play, but you must consider this while creating it so that it does
// not involve damaging players in any way.
var() float TriggerTime;

// optional sound to play when this objective starts, ignored if TriggerTime is 0.0
var() sound TriggerSound;

// sound to play at each 33% that the player completes.  Only the individal player
// can hear this.  Examples would be a small beep for a control panel, a grinding sound
// for a twisting door handle, etc.
var() sound IntervalSound;

// optional sound to play if a player fails to complete the objective while he is
// interacting with it (player is shot, or releases use key).  Does _NOT_ play
// when the end-of-round failed message is sent.
var() sound FailedSound;

// this message will broadcast if the interaction is interrupted before completion. 
var() string InterruptedMessage;

function StartObjective(Pawn Other)
{
	Super.StartObjective(Other);
	if (TriggerTime > 0.0)
	{
		if (TriggerSound != None)
			PlaySound(TriggerSound,,TransientSoundVolume*1.3);
		FOPawn(Other).StartObjective(self,TriggerTime);
		GotoState('PendingCompletion');
	}
	else
		GotoState('CompletedObjective');
}

state PendingCompletion
{
	function OpenObjective(Pawn Other)
	{}

	function BeginState()
	{
		if (!bLinkComplete && StartedMessage != "")
			FragOpsMission(Level.Game).BroadcastLocalizedMessage(class'Interface_ObjectiveMessage',0,None,None,self);
	}
}

function Cancel()
{
	if (FailedSound != None)
		PlaySound(FailedSound,,TransientSoundVolume*1.25);
	if (!bLinkComplete && InterruptedMessage != "")
		FragOpsMission(Level.Game).BroadcastLocalizedMessage(class'Interface_ObjectiveMessage',8,None,None,self);
	Reset();
}

event UsedBy(Pawn Other)
{
	OpenObjective(Other);
}

simulated function PlayIntervalSound()
{
	if (IntervalSound != None)
		PlaySound(IntervalSound,,TransientSoundVolume*1.3);
}

defaultproperties
{
	TriggerTime=0.000
	StartedMessage="Mercenaries are accessing the control panel!"
	FailedSound=Sound'FOWeaponSounds.Objectives.PanelFailed'
	TriggerSound=Sound'FOWeaponSounds.Objectives.PanelStart'
	IntervalSound=Sound'FOWeaponSounds.Objectives.NiceBeep'
	RemoteRole=ROLE_SimulatedProxy
}