//=============================================================================
// Frag.Ops  2003 Matt 'SquirrelZero' Farber
//=============================================================================
// Obj_LAWObjective.uc
// Places a LAW objective in your map.  This will cause the game to give one or
// more random players on ObjectiveTeam a LAW each time a new round begins.
// Mappers can set this to either a StaticMesh or SkeletalMesh
//=============================================================================
class Obj_LAWObjective extends FragOps_ChainableObjective
	placeable;

// Karma Object type
enum EKarmaType
{
	KLead,
	KSteel,
	KRubber,
	KWood,
	KFeather
};

// ChunkKarma, can use karma for chunks instead
struct ChunkKarmaProps
{
	var() bool bUseKarmaInstead;
	var() EKarmaType KarmaType;
};

// Chunkrecord
struct ChunkRecord
{
	var() StaticMesh ChunkMesh;
	var() editinline float ChunkScale;
	var() Sound ChunkBounceSound;
	var() float ChunkVelocity;
	var() editinline ChunkKarmaProps ChunkKarma;
};

// these chunks will fly out everywhere when objective is destroyed
var() editinline array<ChunkRecord> ExplodedChunks;

// optional effects to spawn when objective is destroyed, keep these
// clean and fast, since we are already spawning a LAW explosion
// from the missile.
var() class<Emitter> ExplodedEmitter;
var() class<xEmitter> ExplodedXEmitter;

// mesh to set to when destroyed
var() StaticMesh DestroyedMesh;

var bool bAddedLAWCount;
var StaticMesh OrigMesh;

function PostBeginPlay()
{
	Super.PostBeginPlay();
	if (!bAddedLAWCount)
	{
		bAddedLAWCount = true;
		FragOpsMission(Level.Game).LAWCount[ObjectiveTeam] += 1;
	}
	OrigMesh = StaticMesh;
}

function TakeDamage(int Damage, Pawn instigatedBy, Vector HitLocation, Vector Momentum, class<DamageType> damageType)
{
	local vector start, newstart;
	local rotator rot;
	local int i;
	local Effect_LAWChunk NewChunk;
	local Effect_LAWEmitterSpawner LAWEmitterSpawner;

	if (DamageType.Name != 'DamType_LAW' || bLinkComplete)
		return;

	SetStaticMesh(None);
	SetCollision(false,false,false);
	
	// spawn the explosion emitter
	if (ExplodedEmitter != None || ExplodedXEmitter != None)
	{
		LAWEmitterSpawner = Spawn(class'Effect_LAWEmitterSpawner',,, Location, rotator(vect(0,0,1)));
		LAWEmitterSpawner.EmitterClass = ExplodedEmitter;
		LAWEmitterSpawner.XEmitterClass = ExplodedXEmitter;
		LAWEmitterSpawner.DoSpawn();
	}

	// complete
	Completed(instigatedBy);

	if (DestroyedMesh != None)
	{
		NewChunk = Spawn(class'Effect_LAWChunk',,, Location, Rotation);
		if (NewChunk != None)
		{
			NewChunk.SetStaticMesh(DestroyedMesh);
			NewChunk.SetDrawScale(DrawScale);
		}
		return;
	}

	// spawn some chunks
	start = Location + 10 * vect(0,0,1);
	for (i=0;i<ExplodedChunks.Length;i++)
	{
		rot = rotator(vect(0,0,1));
		rot.yaw += FRand()*32000-16000;
		rot.pitch += FRand()*32000-16000;
		rot.roll += FRand()*32000-16000;
		NewStart.X = (Start.X - 10) + (FRand()*20);
		NewStart.Y = (Start.Y - 10) + (FRand()*20);
		NewChunk = Spawn(class'Effect_LAWChunk',,, Start, rot);
		if (ExplodedChunks[i].ChunkMesh != None && NewChunk != None)
		{
			NewChunk.SetStaticMesh(ExplodedChunks[i].ChunkMesh);
			NewChunk.SetDrawScale(ExplodedChunks[i].ChunkScale);
			NewChunk.ImpactSound = ExplodedChunks[i].ChunkBounceSound;
			if (ExplodedChunks[i].ChunkVelocity >= 1.0)
				NewChunk.Speed = ExplodedChunks[i].ChunkVelocity;
			NewChunk.SetKarma(ExplodedChunks[i].ChunkKarma.bUseKarmaInstead,int(ExplodedChunks[i].ChunkKarma.KarmaType));
		}
	}
}

function Reset()
{
	Super.Reset();
	SetStaticMesh(OrigMesh);
	SetCollision(true,true,true);
}

defaultproperties
{
	bHidden=false
	bWorldGeometry=true
	bCollideWorld=false
	bCollideActors=true
	bBlockPlayers=true
	bBlockActors=true
	bCollideWhenPlacing=false
	bCanBeDamaged=true
	bProjTarget=true
	bAcceptsProjectors=true
	bAddedLAWCount=false
	bAlwaysRelevant=true
	CompletedSound=None
	DrawType=DT_StaticMesh
	StaticMesh=StaticMesh'V-Meshes3.FuelTankDual'
	DestroyedMesh=None
}