//=============================================================================
// Frag.Ops  2003 Matt 'SquirrelZero' Farber
//=============================================================================
// Obj_RetrievalObjective.uc
// Retrieval Objective.  Works in conjunction with an Objective_RetrievalVolume
// to create a CTF-style objective play.  In order for this to work, you must
// set the RetrievalVolume's actor Tag to the same Tag as the actual
// Obj_RetrievalObjective.  Otherwise the game will not know that the player
// is carrying the correct item.  You can have multiple RetrievalVolumes for
// the same item as long as their Tags are the same.
// 
// StartedMessage will broadcast when a player picks the objective up for the
// first time, and ONLY the first time.
// CompletedMessage will broadcast when a player retrieves the objective.
//=============================================================================
class Obj_RetrievalObjective extends FragOps_ChainableObjective
	placeable;

// message to broadcast when a player drops the objective
var() string DroppedMessage;

// only announce pickup once
var() bool bAnnouncePickupOnce;

var() sound PickupSound;

var vector SpawnLocation;
var rotator SpawnRotation;
var bool bInitialBroadcast;
var Pawn Carrier;

function PostBeginPlay()
{
	Super.PostBeginPlay();
	bInitialBroadcast = false;
	SpawnLocation = Location;
	SpawnRotation = Rotation;
}

event Touch(Actor Other)
{
	if (FOPawn(Other) == None || bHidden || bCompleted || bLinkComplete || Carrier != None)
		return;

	OpenObjective(Pawn(Other));
}

// this objective was completed, so player should no longer be carrying it
function Completed(Pawn Other)
{
	Super.Completed(Other);

	if (FOPawn(Other).War_CarriedObjective == self)
		FragOps_PRI(Other.PlayerReplicationInfo).bHasObjective = false;
}

// code for picking object up
function StartObjective(Pawn Other)
{
	Super.StartObjective(Other);
	Carrier = Other;
	bHidden = true;
	SetPhysics(PHYS_None);
	SetCollision(true,false,false);
	bCollideWorld = false;
	SetLocation(Other.Location - (vector(Other.Rotation)*Other.CollisionRadius*0.6));
	SetBase(Other);
	FOPawn(Other).CarriedObjective = self;
	if (Other.PlayerReplicationInfo != None)
		FragOps_PRI(Other.PlayerReplicationInfo).bHasObjective = true;

	PlaySound(PickupSound,,TransientSoundVolume*1.3);

	if (!bInitialBroadcast || !bAnnouncePickupOnce && StartedMessage != "")
	{
		FragOpsMission(Level.Game).BroadcastLocalizedMessage(class'Interface_ObjectiveMessage',0,None,None,self);
		bInitialBroadcast = true;
	}
}

// for falling objective
function HitWall(vector HitNormal, Actor Wall)
{
	local float Speed;
	
	SetPhysics(PHYS_Falling);
	Velocity = 0.4 * (Velocity - 2.0*HitNormal*(Velocity dot HitNormal));
	Speed = VSize(Velocity);

	if (Speed < 30)
		SetPhysics(PHYS_None);
}

function Reset()
{
	Super.Reset();
	bHidden = false;
	bCollideWorld = false;
	SetPhysics(PHYS_None);
	SetCollision(true,false,false);
	SetLocation(SpawnLocation);
	SetRotation(SpawnRotation);
	bInitialBroadcast = false;
	if (Carrier != None)
		Carrier.DetachFromBone(self);
	Carrier = None;
}

defaultproperties
{
	StartedMessage="Mercenaries are stealing the objective!"
	FailedMessage="Mercenaries failed their objective!"
	CompletedMessage="Mercenaries stole the objective!"
	DroppedMessage="Mercenaries dropped the objective!"
	
	Physics=PHYS_None
	DrawType=DT_StaticMesh
	StaticMesh=StaticMesh'dessertstrike2.paper2'
	bHidden=false
	bWorldGeometry=false
	bCollideWorld=false
	bCollideActors=true
	bBlockPlayers=false
	bBlockActors=false
	bCollideWhenPlacing=false
	bAcceptsProjectors=true
	bAlwaysRelevant=true
	bBounce=true
	bReplicateMovement=true
	bUpdateSimulatedPosition=false
	NetPriority=2.0
	RemoteRole=ROLE_DumbProxy
	CollisionRadius=1.0
	CollisionHeight=1.0
}