//=============================================================================
// Frag.Ops  2004 Matt 'SquirrelZero' Farber
//=============================================================================
// The base class for all our vehicle weapons.  Our tracer and hit effect stuff
// is different, same with the way we do our bullet traces.  Much of this is
// similar to FireMode_Base.uc, and follows the same realism model for bullet
// velocity, damage and height loss, etc.  Also adds a similar recoil system.
//=============================================================================
class War_FOVehicleWeapon extends ONSWeapon;

// shells
var() name ShellBone;
var() class<Effect_ShellEjectorFirst> ShellEjectorClass;
var Effect_ShellEjectorFirst ShellEjector;

// flash
var() class<Actor> NewFlashEmitterClass;
var Actor NewFlashEmitter;

// smoke
var() class<Actor> NewSmokeEmitterClass;
var Actor NewSmokeEmitter;

// hit effects, mine are better! :P
var() class<Effect_BulletHit> BulletHitEffectClass;

// minimum bullet damage
var int MinDamage;

// range of this weapon, damage shrinkage (bullet velocity damage ticking)
var float WeaponRange;
var float WeaponDamageShrink;
var float BulletHeightLoss;

// recoil
var float RecoilSize;
var float InitialRecoil;
var bool bHasRecoiled;

// other stuff
var float LastHeadShotTime, LastBodyHitTime;

// ammo
var int AmmoAmount;
var const int MaxAmmo;
var int NumClips;
var const int MaxClips;

// reloading
var const float ReloadTime;
var sound ReloadBeginSound, ReloadMidSound, ReloadEndSound;
var bool bReloaded;

// HUD
var string FireType, AmmoType;

// team
var int TeamIndex;

// restore ammunition once every 2 waves
var bool bWaveAlternate;

replication
{
	reliable if (Role < ROLE_Authority)
		Reload;
	
	reliable if (Role == ROLE_Authority)
		DoRecoil;

	reliable if (bNetDirty && Role == ROLE_Authority)
		AmmoAmount, NumClips;
}

// added bone-placed shell ejections
simulated function InitEffects()
{
	// don't even spawn on server
	if (Level.NetMode == NM_DedicatedServer)
		return;

	Super.InitEffects();

	if ((NewFlashEmitterClass != None) && ((NewFlashEmitter == None) || NewFlashEmitter.bDeleteMe))
	{
		if (NewFlashEmitter != None)
			NewFlashEmitter.Destroy();
		NewFlashEmitter = Spawn(NewFlashEmitterClass,self);
		AttachToBone(NewFlashEmitter,WeaponFireAttachmentBone);
	}

	if ((NewSmokeEmitterClass != None) && ((NewSmokeEmitter == None) || NewSmokeEmitter.bDeleteMe))
	{
		if (NewSmokeEmitter != None)
			NewSmokeEmitter.Destroy();
		NewSmokeEmitter = Spawn(NewSmokeEmitterClass,self);
		AttachToBone(NewSmokeEmitter,WeaponFireAttachmentBone);
	}
	
	if ((ShellEjectorClass != None) && ((ShellEjector == None) || ShellEjector.bDeleteMe))
	{
		if (ShellEjector != None)
			ShellEjector.Destroy();
		ShellEjector = Spawn(ShellEjectorClass,self);
		ShellEjector.InitEjector();
	}
}

simulated function PlayReload()
{
	if (IsInState('Reloading') || AmmoAmount == MaxAmmo || NumClips <= 0)
		return;
	
	Reload();
	if (Role < ROLE_Authority)
		GotoState('Reloading','Begin');
}

function Reload()
{
	if (IsInState('Reloading') || AmmoAmount == MaxAmmo || NumClips <= 0)
		return;
	
	GotoState('Reloading','Begin');
}

simulated state Reloading
{
	ignores Fire, TraceFire, FlashMuzzleFlash;

	simulated function BeginState()
	{
		Vehicle(Owner).Driver.SetAnimAction('Weapon_Switch');
		if (Role == ROLE_Authority)
			NumClips -=1;
	}

	simulated function EndState()
	{
		if (Role == ROLE_Authority)
		{
			if (bReloaded)
				AmmoAmount = MaxAmmo;
			else
				NumClips = Clamp(NumClips+1, 0, MaxClips);
		}
	}
Begin:
	bReloaded = false;
	PlayOwnedSound(ReloadBeginSound,,,,100);
	Sleep(ReloadTime*0.25);
	Vehicle(Owner).Driver.SetAnimAction('vehicle_reload');
	PlayOwnedSound(ReloadMidSound,,,,100);
	Sleep(ReloadTime*0.5);
	Vehicle(Owner).Driver.SetAnimAction('Weapon_Switch');
	PlayOwnedSound(ReloadEndSound,,0.5*TransientSoundVolume,,100);
	Sleep(ReloadTime*0.25);
	bReloaded = true;
	if (bInstantFire)
		GotoState('InstantFireMode');
	else
		GotoState('ProjectileFireMode');
}

event bool AttemptFire(Controller C, bool bAltFire)
{
  	if (Role != ROLE_Authority || bForceCenterAim || ((AmmoAmount <= 0 && !bIsAltFire) || IsInState('Reloading')))
		return False;

	if (FireCountdown <= 0)
	{
		CalcWeaponFire();
		if (bCorrectAim)
			WeaponFireRotation = AdjustAim(bAltFire);
		if (Spread > 0)
			WeaponFireRotation = rotator(vector(WeaponFireRotation) + VRand()*FRand()*Spread);

        	DualFireOffset *= -1;

		Instigator.MakeNoise(1.0);
		if (bAltFire)
		{
			FireCountdown = AltFireInterval;
			AltFire(C);
		}
		else
		{
			FireCountdown = FireInterval;
			Fire(C);
		}
		AimLockReleaseTime = Level.TimeSeconds + FireCountdown * FireIntervalAimLock;
		return true;
	}

	return False;
}

simulated function ShakeView()
{
	local War_FOPlayer P;

	if ((AmmoAmount <= 0 && !bIsAltFire) || IsInState('Reloading'))
		return;

	if (Instigator == None)
		return;

	P = War_FOPlayer(Instigator.Controller);
	if (P != None)
		P.VehicleShakeView(ShakeRotMag, ShakeRotRate, ShakeRotTime, ShakeOffsetMag, ShakeOffsetRate, ShakeOffsetTime);
}

simulated function ClientStartFire(Controller C, bool bAltFire)
{
	bIsAltFire = bAltFire;

	if (FireCountdown <= 0)
	{
		if (bIsRepeatingFF)
		{
			if (bIsAltFire)
				ClientPlayForceFeedback( AltFireForce );
			else
				ClientPlayForceFeedback( FireForce );
		}
		OwnerEffects();
	}
}

simulated event OwnerEffects()
{
	if ((AmmoAmount <= 0 && !bIsAltFire) || IsInState('Reloading'))
	{
		if (Role < ROLE_Authority)
		{
			if (bIsAltFire)
				FireCountdown = AltFireInterval;
			else
				FireCountdown = FireInterval;

			AimLockReleaseTime = Level.TimeSeconds + FireCountdown * FireIntervalAimLock;
		}
		return;
	}

	if (!bIsRepeatingFF)
	{
		if (bIsAltFire)
			ClientPlayForceFeedback(AltFireForce);
		else
			ClientPlayForceFeedback(FireForce);
	}
	ShakeView();

	if (Role < ROLE_Authority)
	{
		if (bIsAltFire)
			FireCountdown = AltFireInterval;
		else
			FireCountdown = FireInterval;

		AimLockReleaseTime = Level.TimeSeconds + FireCountdown * FireIntervalAimLock;

		FlashMuzzleFlash();

		if (AmbientEffectEmitter != None)
			AmbientEffectEmitter.SetEmitterStatus(true);

		// Play firing noise
		if (!bAmbientFireSound)
		{
			if (bIsAltFire)
				PlaySound(AltFireSoundClass, SLOT_Interact, FireSoundVolume/255.0,, AltFireSoundRadius,, false);
			else
				PlaySound(FireSoundClass, SLOT_Interact, FireSoundVolume/255.0,, FireSoundRadius,, false);
		}
	}
}

state InstantFireMode
{
	function Fire(Controller C)
	{
		if (IsInState('Reloading') || AmmoAmount <= 0)
			return;
		
		FlashMuzzleFlash();

		if (Role == ROLE_Authority)
			ConsumeAmmo(1);

		if (AmbientEffectEmitter != None)
			AmbientEffectEmitter.SetEmitterStatus(true);

		// Play firing noise
		if (bAmbientFireSound)
			AmbientSound = FireSoundClass;
		else
			PlayOwnedSound(FireSoundClass, SLOT_None, FireSoundVolume/255.0,, FireSoundRadius, (FireSoundPitch-0.04)+(FRand()*0.08), False);

		TraceFire(WeaponFireLocation, WeaponFireRotation);
	}
}

function ConsumeAmmo(int AmmoLoad)
{
	AmmoAmount = Max(AmmoAmount - AmmoLoad, 0);
}

// just like the tracing in FireMode_Base
function TraceFire(Vector Start, Rotator Dir)
{
	local Vector End, XVec;
	local Rotator R, Aim;
	local Vector HitLocation, HitNormal, HitRay;
	local Actor Other;
	local float BulletDistance;
	local float FODamage;
	local int Damage, i;
	local vector BoneTestLocation;
	local name HitBone;
	local Material SurfaceMat;
	local bool bLastShot, bContinuedShot;
	local Vehicle VehicleInstigator, FoundVehicle;
	local FOPawn FP;

	bLastShot = false;

	MaxRange();

	// for accuracy stats
	FragOps_PRI(Pawn(Owner).PlayerReplicationInfo).AddShot();

	while (true)
	{
		if (bLastShot || bContinuedShot)
			Dir = Aim;

		XVec = vector(Dir);
		End = Start + TraceRange * XVec;
	
		//skip past vehicle driver
		VehicleInstigator = Vehicle(Instigator);
		if (VehicleInstigator != None && VehicleInstigator.Driver != None)
		{
			VehicleInstigator.Driver.bBlockZeroExtentTraces = false;
			Other = Trace(HitLocation, HitNormal, End, Start, true,, SurfaceMat);
			VehicleInstigator.Driver.bBlockZeroExtentTraces = true;
		}
		else
			Other = Trace(HitLocation, HitNormal, End, Start, true,, SurfaceMat);

		// recoil weapon
		if (!bLastShot && !bContinuedShot && RecoilSize > 0)
			DoRecoil();

		BulletDistance = VSize(Start - HitLocation);
		
		if (Other != None && (Other != Instigator))
		{
			Damage = (DamageMin + Rand(DamageMax - DamageMin));
			
			// vehicle damage
			if (War_FOVehicleWheeled(Other) != None)
			{
				FoundVehicle = Vehicle(Other);
				Other = War_FOVehicleWheeled(FoundVehicle).GetHitPawn(HitLocation, R);
				if (Other == None)
				{
					FoundVehicle.TakeDamage(Damage, Pawn(Owner), HitLocation, XVec, DamageType);
					break;
				}
			}
			else if (War_FOVehiclePawn(Other) != None)
			{
				FoundVehicle = Vehicle(Other);
				Other = War_FOVehiclePawn(FoundVehicle).GetHitPawn(HitLocation, R);
				if (Other == None)
				{
					FoundVehicle.TakeDamage(Damage, Pawn(Owner), HitLocation, XVec, DamageType);
					break;
				}
			}
			
			FP = FOPawn(Other);
			if (FP != None)
			{
				// for accuracy stats
				if (FP.Health > 0 && RecoilSize > 0)
					FragOps_PRI(Pawn(Owner).PlayerReplicationInfo).AddHit();

				Damage = (DamageMin + Rand(DamageMax - DamageMin));
				AdjustDamage(Damage);
				if (BulletDistance > WeaponRange)
				{
					FODamage = BulletDistance - WeaponRange;
					FODamage = FODamage / WeaponDamageShrink;
					Damage = Damage - int(FODamage);
				}

				// this new method works much better.
				BoneTestLocation = HitLocation + (Other.CollisionRadius*0.12)*XVec;
				HitRay = Normal(XVec);
								
				// find the closest bone.
				HitBone = 'none';
				// scale the head bone location to the center of the face
				if (Pawn(Other).IsHeadShot(HitLocation, XVec, 1.0))
					HitBone = 'headshot';
				else
				{
					for (i=0;i<class'FireMode_Base'.Default.BoneData.Length;i++)
					{
						if (FP.IsBoneHit(HitLocation, XVec, 1.0, class'FireMode_Base'.Default.BoneData[i].BoneName, class'FireMode_Base'.Default.BoneData[i].BoneRadius, class'FireMode_Base'.Default.BoneData[i].BoneHeight))
						{
							HitBone = class'FireMode_Base'.Default.BoneData[i].BoneName;
							break;
						}
					}
				}

				// if we hit no bone, move the shot beyond the player's collision radius and continue down fire path
				if (HitBone == 'none' || HitBone == '' && !bContinuedShot)
				{
					Aim = rotator(End - Start);
					Start = HitLocation + 0.5*(Other.CollisionRadius*vector(Aim));
					bContinuedShot = true;
				}
				else if (HitBone != 'none' && HitBone != '')
				{
					CalcDamage(Damage, HitBone, Other);

					if (HitBone == 'headshot' && FP.Health > 0)
					{					
						if (AllowHitSound(Pawn(Other)))
							FP.PlayHeadShotSound();
					}
					else if (HitBone != 'headshot' && AllowHitSound(Pawn(Other)))
						FP.PlayBodyHitSound();
					if (Damage < MinDamage)
						Damage = MinDamage;

					FP.LastBone = HitBone;
					Other.TakeDamage(Damage, Instigator, HitLocation, XVec, DamageType);
					break;
				}
				else
					break;
			}
			else if (Other.Default.bCanBeDamaged || Other.Physics == PHYS_Karma || !Other.bWorldGeometry)
			{
				if (Projectile(Other) == None)
					BulletHitEffect(Start, R, HitLocation, HitNormal, SurfaceMat);
				Other.TakeDamage((DamageMin + Rand(DamageMax - DamageMin)), Pawn(Owner), HitLocation, XVec, DamageType);
				break;
			}
			else
			{
				BulletHitEffect(Start, R, HitLocation, HitNormal, SurfaceMat);
				break;
			}
		}
		else if (Other == None && !bLastShot && RecoilSize > 0) // simulate gravity by pointing bullets downwards
		{
			Aim = rotator(End - Start);
			Start = End;
			Aim.Pitch -= BulletHeightLoss;
			bLastShot = true;
		}
		else
			break;
	}
	NetUpdateTime = Level.TimeSeconds - 1;
}

function bool AllowHitSound(Pawn P)
{
	return ((FragOpsGame(Level.Game).FriendlyFireScale > 0.0) || (P.PlayerReplicationInfo == None) || (Pawn(Owner).PlayerReplicationInfo.Team != P.PlayerReplicationInfo.Team));
}

function AdjustDamage(out int Damage);

function CalcDamage(out int Damage, name HitBone, actor VActor)
{
	local int VictimTeam;
	VictimTeam = 2;
	if (FOPawn(VActor) != None)
	{
		if ((FOPawn(VActor).Health > 0) && (FOPawn(VActor).PlayerReplicationInfo.Team != None) )
			VictimTeam = FOPawn(VActor).PlayerReplicationInfo.Team.TeamIndex;

	}
	switch(HitBone)
	{
	// head hits, 1.55x dmg
		case 'headshot':
			Damage *= 1.9;
			break;

	// chest, shoulder hits, normal dmg
		case 'spine':
			Damage *= 1.0;
			break;

		case 'pelvis':
			Damage *= 1.0;
			break;

		case 'rshoulder':
			Damage *= 1.1;
			break;

		case 'lshoulder':
			Damage *= 1.1;
			break;

	// arms, thighs, 0.75x dmg
		case 'rfarm':
			Damage *= 0.75;
			break;

		case 'lfarm':
			Damage *= 0.75;
			break;

		case 'lthigh':
			Damage *= 0.75;
			break;

		case 'rthigh':
			Damage *= 0.75;
			break;

	// feet, hands, 0.25x dmg
		case 'leftfoot':
			Damage *= 0.3;
			break;

		case 'rightfoot':
			Damage *= 0.3;
			break;

		case 'lefthand':
			Damage *= 0.35;
			break;

		case 'righthand':
			Damage *= 0.35;
			break;

	// otherwise we default to normal, eventually i want to change this so that shots can pass through
	// the invisible collision area around the head and in front of the feet.
		case 'none':
			Damage *= 1.1;
	}
}

function BulletHitEffect(Vector Start, Rotator Dir, Vector HitLocation, Vector HitNormal, Material SurfaceMat)
{
	local Effect_BulletHit BulletHit;

	BulletHit = spawn(BulletHitEffectClass, None,, Start, Dir);

	// set self as the tracer actor
	War_Effect_BulletHit(BulletHit).TracerActor = self;
	BulletHit.ImpactAt(HitLocation, HitNormal, SurfaceMat, Vehicle(Owner).Driver, DamageType);
}

simulated function DoRecoil()
{
	if (Owner == None || Pawn(Owner) == None || Pawn(Owner).Controller == None)
	{
		bHasRecoiled = false;
		return;
	}

	// smoothing
	bHasRecoiled = true;
	InitialRecoil = RecoilSize;
}

simulated function Tick(float dt)
{
	local float PrevRecoil;

	Super.Tick(dt);

	if (Level.NetMode == NM_DedicatedServer || Owner == None || Pawn(Owner) == None || Pawn(Owner).Controller == None)
		return;

	// recoil smoothing
	if (bHasRecoiled)
	{
		PrevRecoil = InitialRecoil;
		InitialRecoil -= (PrevRecoil*dt)*19.0;

		if (InitialRecoil <= 0.0)
			bHasRecoiled = false;
	}
}

simulated event FlashMuzzleFlash()
{
	if (Role == ROLE_Authority)
	{
		FlashCount++;
		NetUpdateTime = Level.TimeSeconds - 1;
	}
	else
	        CalcWeaponFire();

	EjectShell();
	if (NewFlashEmitter != None)
		NewFlashEmitter.Trigger(self, Pawn(Owner));
	if (NewSmokeEmitter != None)
		NewSmokeEmitter.Trigger(self, Pawn(Owner));
}

simulated function EjectShell()
{
	if (ShellEjector == None)
		return;

	ShellEjector.Trigger(self, Pawn(Owner));
}

// limityaw.. necessary for us
simulated function int LimitYaw(int InYaw, rotator CurrentRotation)
{
	local int VehicleYaw;

	VehicleYaw = CurrentRotation.Yaw & 65535;
	InYaw = InYaw & 65535;

	//VehicleYaw = CurrentRotation.Yaw;
	if ((InYaw ClockwiseFrom (VehicleYaw + YawEndConstraint)) && (InYaw ClockwiseFrom VehicleYaw))
		return VehicleYaw + YawEndConstraint;
	else if (!(InYaw ClockwiseFrom (VehicleYaw + YawStartConstraint)) && !(InYaw ClockwiseFrom VehicleYaw))
		return VehicleYaw + YawStartConstraint;
	else
		return InYaw;
}

simulated function DestroyEffects()
{
	Super.DestroyEffects();

	if (ShellEjector != None)
		ShellEjector.Destroy();
	if (NewFlashEmitter != None)
		NewFlashEmitter.Destroy();
	if (NewSmokeEmitter != None)
		NewSmokeEmitter.Destroy();
}

simulated function SetOverlayMaterial(Material mat, float time, bool bOverride)
{
}

defaultproperties
{
	bInstantRotation=true
	bAimable=true
	bUseCollisionStaticMesh=true
	BulletHeightLoss=500
	BulletHitEffectClass=class'War_Effect_BulletHit'
	MinDamage=15

	// tweaks
	MaxLights=3
}