//=============================================================================
// Frag.Ops  2004 Matt 'SquirrelZero' Farber
//=============================================================================
// Base objective class for War gametype.  Should bear a resemblance to the
// Mission chainable objective, but actual execution is much different.
//=============================================================================
class War_Objective extends Actor
	abstract
	noteditinlinenew
	notplaceable;

// information about this objective
struct ObjectiveInfo
{
	var() Material StartedPicture;
	var() Material CompletedPicture;
	var() string StartedBriefing;
	var() string CompletedBriefing;
};

// split information by team
var() editinline ObjectiveInfo AttackTeam_ObjectiveInfo;
var() editinline ObjectiveInfo DefendTeam_ObjectiveInfo;

// sounds, self explanatory
var() Sound StartedSound, CompletedSound;

// events
var() name CompletedEvent;

// can complete out of order if true
var() bool bCanCompleteInAnyOrder;

// completing this objective is optional
var() bool bCompletionOptional;

// disable this objective after triggering another in the same link
var() bool bDisableAfterLinkTrigger;

// gameinfo
var FragOpsWar WarGame;

// attacking team, set by levelinfo
var byte AttackingTeam;

// link #
var byte LinkNumber;

// completed?
var bool bCompleted;

// vehicles allowed to use this objective?
var bool bVehiclesAllowed;

// sounds
var Sound AnnounceLevels[3];

function Cancel();

replication
{
	reliable if (bNetDirty && ROLE == ROLE_Authority)
		bCompleted;
}

// called when someone attempts to access this objective
function bool OpenObjective(Pawn Other)
{
	if (!WarGame.CanTrigger(self,Other))
		return false;
	return true;
}

// the initial state for this objective.  Basically the "idle" state
state IdleObjective
{
	function Cancel()
	{}

	function BeginState();
}

// completed state
state CompletedObjective
{
	function bool OpenObjective(Pawn Other)
	{
		return false;
	}

	function Cancel()
	{
	}

	function BeginState()
	{
		if (CompletedSound != None)
			PlaySound(CompletedSound,,TransientSoundVolume*1.3);

		if (AttackingTeam == 1)
		{
			if (bCanCompleteInAnyOrder)
				WarGame.WarLevelInfo.LastSCaptureLink = LinkNumber;
			else
				WarGame.WarLevelInfo.LastSObjLink = LinkNumber;
		}
		else
		{
			if (bCanCompleteInAnyOrder)
				WarGame.WarLevelInfo.LastMCaptureLink = LinkNumber;
			else
				WarGame.WarLevelInfo.LastMObjLink = LinkNumber;
		}

		SetCollision(false,false,false);
	}

	function EndState();

Begin:
	WarGame.CheckLinks(AttackingTeam);
	Sleep(0.01);
	PrintObjectiveSplash(AttackTeam_ObjectiveInfo.CompletedBriefing, DefendTeam_ObjectiveInfo.CompletedBriefing);
}

// broadcast our unique objective message
function PrintObjectiveSplash(string AttackSplashString, string DefendSplashString)
{
	local Controller C;
	local Sound ObjLevelSound;

	if (WarGame.bRoundEnded)
		return;
	
	// get a sound to play
	if (!bCompleted)
		ObjLevelSound = None;
	else if (bCanCompleteInAnyOrder)
		ObjLevelSound = AnnounceLevels[0];
	else
	{
		if (AttackingTeam == 1)
		{
			if (LinkNumber+1 == WarGame.WarLevelInfo.S_Obj.Length)
				ObjLevelSound = AnnounceLevels[2];
			else if (float(LinkNumber+1)/float(WarGame.WarLevelInfo.S_Obj.Length) > 0.5)
				ObjLevelSound = AnnounceLevels[1];
			else
				ObjLevelSound = AnnounceLevels[0];
		}
		else
		{
			if (LinkNumber+1 == WarGame.WarLevelInfo.M_Obj.Length)
				ObjLevelSound = AnnounceLevels[2];
			else if (float(LinkNumber+1)/float(WarGame.WarLevelInfo.M_Obj.Length) > 0.5)
				ObjLevelSound = AnnounceLevels[1];
			else
				ObjLevelSound = AnnounceLevels[0];
		}
	}

	for (C=Level.ControllerList;C!=None;C=C.NextController)
	{
		if (FOBot(C) != None || C.Pawn == None)
			continue;

		if (C.PlayerReplicationInfo.Team.TeamIndex == AttackingTeam && AttackSplashString != "")
		{
			if (bCompleted)
				War_FOPlayer(C).ClientSplashObjective(AttackSplashString, AttackTeam_ObjectiveInfo.CompletedPicture, ObjLevelSound);
			else
				War_FOPlayer(C).ClientSplashObjective(AttackSplashString, AttackTeam_ObjectiveInfo.StartedPicture, ObjLevelSound);
		}
		else if (C.PlayerReplicationInfo.Team.TeamIndex != AttackingTeam && DefendSplashString != "")
		{
			if (bCompleted)
				War_FOPlayer(C).ClientSplashObjective(DefendSplashString, DefendTeam_ObjectiveInfo.CompletedPicture, None);
			else
				War_FOPlayer(C).ClientSplashObjective(DefendSplashString, DefendTeam_ObjectiveInfo.StartedPicture, None);
		}
	}
}

// complete this objective
function Completed(Pawn Other)
{
	local Controller C;
	
	if (FragOpsGame(Level.Game).bRoundEnded || bCompleted || FragOps_GRI(Level.Game.GameReplicationInfo).bPreRound)
		return;

	bCompleted = true;

	// send immediately
	NetUpdateTime = Level.TimeSeconds - 1;

	// update radar objective
	if (!bCanCompleteInAnyOrder)
	{
		for (C=Level.ControllerList;C!=None;C=C.nextController)
		{
			if (FOBot(C) != None || C.PlayerReplicationInfo == None || C.PlayerReplicationInfo.Team == None || !C.bIsPlayer || FOPlayer(C).CriticalReplicationInfo.bWaitingForTeam)
				continue;
			
			if (War_FOPlayer(C).Objective == self)
			{
				if (AttackingTeam == 1 && LinkNumber != WarGame.WarLevelInfo.SpecOps_Objectives.Length-1)
					War_FOPlayer(C).Objective = WarGame.WarLevelInfo.SpecOps_Objectives[LinkNumber+1].Objective;
				else if (AttackingTeam == 0 && LinkNumber != WarGame.WarLevelInfo.Merc_Objectives.Length-1)
					War_FOPlayer(C).Objective = WarGame.WarLevelInfo.Merc_Objectives[LinkNumber+1].Objective;
				else
					War_FOPlayer(C).Objective = None;
			}
		}
	}

	// award XP
	if (Other != None && Other.PlayerReplicationInfo != None)
		War_PRI(Other.PlayerReplicationInfo).AddXP(WarGame.ObjectiveXP);

	// trigger completed event
	TriggerEvent(CompletedEvent, self, Other);

	// go to completed state
	GotoState('CompletedObjective','Begin');
}

// resets to the initial settings
function Reset()
{
	bCompleted = false;
	GotoState(InitialState);
	if (WarGame == None && Level.Game != None)
		WarGame = FragOpsWar(Level.Game);
}

defaultproperties
{
	Texture=Texture'FOEditorTextures.Objectives.ObjectiveMain'
	InitialState=IdleObjective
	bDisableAfterLinkTrigger=true
	
	// announcements
	AnnounceLevels(0)=Sound'FOMaleVoices.AmericanExtras.GoodWorkKeepMoving'
	AnnounceLevels(1)=Sound'FOMaleVoices.AmericanExtras.AlmostThere'
	AnnounceLevels(2)=Sound'FOMaleVoices.AmericanExtras.LetsFinishThis'

	// settings
	bHidden=true
	bHiddenEd=false
	bEdShouldSnap=true
	bBlockPlayers=false
	bBlockActors=false
	bCollideActors=false
	bCollideWorld=false
	bCollideWhenPlacing=false
	bAlwaysRelevant=true
	bCompleted=false
	CollisionRadius=30.000000
	CollisionHeight=30.000000
	bUseCylinderCollision=true
	RemoteRole=ROLE_DumbProxy
	bStatic=false
	bNoDelete=true
}