//=============================================================================
// Frag.Ops  2004 Matt 'SquirrelZero' Farber
//=============================================================================
// Just like the one from mission.
//=============================================================================
class War_Objective_Destroyable extends War_Objective
	placeable;

// Karma Object type
enum EKarmaType
{
	KLead,
	KSteel,
	KRubber,
	KWood,
	KFeather
};

// ChunkKarma, can use karma for chunks instead
struct ChunkKarmaProps
{
	var() bool bUseKarmaInstead;
	var() EKarmaType KarmaType;
};

// Chunkrecord
struct ChunkRecord
{
	var() StaticMesh ChunkMesh;
	var() editinline float ChunkScale;
	var() Sound ChunkBounceSound;
	var() float ChunkVelocity;
	var() float ChunkLifeSpan;
	var() editinline ChunkKarmaProps ChunkKarma;
};

// these chunks will fly out everywhere when objective is destroyed
var() editinline array<ChunkRecord> ExplodedChunks;

// optional effects to spawn when objective is destroyed, keep these
// clean and fast, since we are already spawning a LAW explosion
// from the missile.
var() class<Emitter> ExplodedEmitter;
var() class<xEmitter> ExplodedXEmitter;

// mesh to set to when destroyed
var() StaticMesh DestroyedMesh;

// maximum damage this objective can withstand before destruction
var() int MaxDamage;

// location offset to spawn emitter/chunks
var() vector EffectSpawnOffset;

// works just like StartedBriefing in War_LevelInfo, used when objective is first
// triggered
var() string Attack_TriggeredMessage;
var() string Defend_TriggeredMessage;

// view shaking
var() bool bShakeScreen;
var() float ShakeRadius;

// is this objective vulnerable to LAW projectiles?
var() bool bLAWTarget;

var int DamageTaken;
var bool bHasBroadcast;
var StaticMesh OrigMesh;

replication
{
	reliable if (bNetDirty && Role == ROLE_Authority)
		DamageTaken;
}

function PostBeginPlay()
{
	Super.PostBeginPlay();
	OrigMesh = StaticMesh;
}

function TakeDamage(int Damage, Pawn InstigatedBy, Vector HitLocation, Vector Momentum, class<DamageType> damageType)
{
	local vector start, newstart;
	local rotator rot;
	local int i;
	local Effect_LAWChunk NewChunk;
	local Effect_LAWEmitterSpawner LAWEmitterSpawner;

	if (Damage <= 0)
		return;

	if (InstigatedBy == None || !OpenObjective(InstigatedBy))
		return;

	if (DamageType.Name == 'DamType_Machete' || DamageType.Name == 'DamType_AKBayonet' || DamageType.Name == 'DamType_MachetePoison')
		DamageTaken += Damage/3;
	else
		DamageTaken += Damage;

	if (DamageTaken >= (MaxDamage/2) && !bHasBroadcast && DamageTaken < MaxDamage)
	{
		PrintObjectiveSplash(Attack_TriggeredMessage, Defend_TriggeredMessage);
		bHasBroadcast = true;
	}

	if (DamageTaken < MaxDamage)
		return;

	SetStaticMesh(None);
	SetCollision(false,false,false);
	
	// spawn the explosion emitter
	if (ExplodedEmitter != None || ExplodedXEmitter != None)
	{
		LAWEmitterSpawner = Spawn(class'Effect_LAWEmitterSpawner',,, (Location + EffectSpawnOffset), rotator(vect(0,0,1)));
		LAWEmitterSpawner.EmitterClass = ExplodedEmitter;
		LAWEmitterSpawner.XEmitterClass = ExplodedXEmitter;
		LAWEmitterSpawner.DoSpawn();
	}

	if (bShakeScreen)
		ShakeView();

	// complete
	Completed(InstigatedBy);
	
	// spawn some chunks
	Start = Location + EffectSpawnOffset;
	for (i=0;i<ExplodedChunks.Length;i++)
	{
		rot = rotator(vect(0,0,1));
		rot.yaw += FRand()*32000-16000;
		rot.pitch += FRand()*16000-8000;
		rot.roll += FRand()*16000-8000;
		NewStart.Z = Start.Z;
		NewStart.X = (Start.X - 30) + (FRand()*60);
		NewStart.Y = (Start.Y - 30) + (FRand()*60);
		NewChunk = Spawn(class'Effect_LAWChunk',,, NewStart, rot);
		if (ExplodedChunks[i].ChunkMesh != None && NewChunk != None)
		{
			NewChunk.SetStaticMesh(ExplodedChunks[i].ChunkMesh);
			NewChunk.SetDrawScale(ExplodedChunks[i].ChunkScale);
			NewChunk.ImpactSound = ExplodedChunks[i].ChunkBounceSound;
			NewChunk.NewLifeSpan = ExplodedChunks[i].ChunkLifeSpan;
			if (ExplodedChunks[i].ChunkVelocity >= 1.0)
				NewChunk.Speed = ExplodedChunks[i].ChunkVelocity;
			NewChunk.SetKarma(ExplodedChunks[i].ChunkKarma.bUseKarmaInstead,int(ExplodedChunks[i].ChunkKarma.KarmaType));
		}
	}

	// set destroyed mesh
	if (DestroyedMesh != None)
	{
		SetStaticMesh(DestroyedMesh);
		SetCollision(true,true,true);
	}
}

function ShakeView()
{
	local Controller C;
	local PlayerController PC;
	local float Dist, Scale;

	for (C=Level.ControllerList;C!=None;C=C.NextController)
	{
		PC = PlayerController(C);
		if (PC != None && PC.ViewTarget != None)
		{
			Dist = VSize(Location - PC.ViewTarget.Location);
			if (Dist < ShakeRadius * 2.0)
			{
				if (Dist < ShakeRadius)
					Scale = 1.0;
				else
					Scale = (ShakeRadius*2.0 - Dist) / (ShakeRadius);
				FOPlayer(PC).ClientShake(vect(350,350,350)*Scale,vect(9000,9000,9000),3.5, vect(10,10,10)*Scale,vect(2000,2000,2000),3.5);
			}
		}
	}
}

function Reset()
{
	Super.Reset();
	SetStaticMesh(OrigMesh);
	SetCollision(true,true,true);
	DamageTaken = 0;
	bHasBroadcast = false;
}

// the initial state for this objective.  Basically the "idle" state
state IdleObjective
{
	function Cancel()
	{
	}

	function BeginState()
	{
	}
}

defaultproperties
{
	Physics=PHYS_None
	bHidden=false
	bBlockZeroExtentTraces=true
	bBlockNonZeroExtentTraces=true
	bWorldGeometry=true
	bCollideWorld=false
	bCollideActors=true
	bVehiclesAllowed=true
	bBlockPlayers=true
	bBlockActors=true
	bUseDynamicLights=true
	bCollideWhenPlacing=false
	bBlockKarma=true
	bCanBeDamaged=true
	bProjTarget=true
	bAcceptsProjectors=false
	bAlwaysRelevant=true
	bUseCylinderCollision=false
	bLAWTarget=true
	CompletedSound=None
	DrawType=DT_StaticMesh
	StaticMesh=StaticMesh'V-Meshes3.FuelTankDual'
	DestroyedMesh=None
	MaxDamage=800
	CollisionHeight=+000001.000000
	CollisionRadius=+000001.000000
	RemoteRole=ROLE_SimulatedProxy
}