//=============================================================================
// Frag.Ops  2004 Matt 'SquirrelZero' Farber
//=============================================================================
// Base grenade projectile.
//=============================================================================
class Weapon_GrenadeProjectile extends Projectile;

// hit effects 
var class<xEmitter> HitEffectClass;
var float LastSparkTime;

// timer till explode
var float GrenExplodeTimer;

// explosion spawner class
var class<Effect_ExplosionSpawner> ExplosionSpawnerClass;

// damage taken/can take
var int DamageTaken, MaxDamage;

// added gravity pull
var float GravityDampen;

// for adding velocity boosts to grenades
var vector BoostVelocity, LastBoost;

// shake view?
var bool bShakeView;

// water splashes
var sound SplashSound;
var float SplashThreshold;
var bool bHitWater;

replication
{
	reliable if (bNetDirty && (Role == ROLE_Authority))
		BoostVelocity;
}

simulated function PostBeginPlay()
{
	local vector TossVel;

	Super.PostBeginPlay();

	SetOwner(None);
	SetPhysics(PHYS_Falling);
	if (Role == ROLE_Authority)
	{
		TossVel = (Vector(Rotation) * (Speed));
		TossVel.Z += TossZ;
		Velocity = TossVel;
		RandSpin(50000);
		DamageTaken = 0;
		if (Instigator.HeadVolume.bWaterVolume)
		{
			bHitWater = true;
			Velocity = 0.4*Velocity;
		}
	}
	if (Instigator != None)
		InstigatorController = Instigator.Controller;
	
	if (ROLE == ROLE_Authority)
		SetTimer(GrenExplodeTimer, false);
}

// slow grenade on impact, play splash sound effects
simulated event PhysicsVolumeChange(PhysicsVolume NewVolume)
{
	local float SplashVolume;

	Super.PhysicsVolumeChange(NewVolume);
	
	if (!NewVolume.bWaterVolume)
		return;

	if (!bHitWater && Role == ROLE_Authority)
	{
		Velocity *= 0.4;
		Velocity.Z += 60;
		bHitWater = true;
	}
	
	if (Level.NetMode == NM_DedicatedServer)
		return;

	Speed = VSize(Velocity);
	
	SplashVolume = FClamp(Speed/SplashThreshold, 0, 0.77);
	PlaySound(SplashSound, Slot_NONE, SplashVolume,, 500, 0.96+(FRand()*0.08));
	
}

simulated function PostNetReceive()
{
	if (BoostVelocity != LastBoost && BoostVelocity != vect(0,0,0))
	{
		// reset bounce
		bBounce = true;
		LastBoost = BoostVelocity;
		Velocity += BoostVelocity;
	}
}

simulated function Tick(float deltatime)
{
	if (Role == ROLE_Authority && (Velocity.Z > 5 || Velocity.Z < -5) && !PhysicsVolume.bWaterVolume)
	{
		if (Velocity.Z > -1600)
			Velocity.Z -= GravityDampen;
	}
	Super.Tick(deltatime);
}

function Timer()
{
	Explode(Location, vect(0,0,1));
}

simulated function Landed( vector HitNormal )
{
	HitWall(HitNormal, None);
}

simulated function HitWall( vector HitNormal, actor Wall )
{
	SetPhysics(PHYS_Falling);
	Velocity = 0.4 * (Velocity - 2.0*HitNormal*(Velocity dot HitNormal));
	RandSpin(50000);
	Speed = VSize(Velocity);

	if (Speed < 30)
		bBounce = false;
	else
	{
		if (Level.NetMode != NM_DedicatedServer)
		{
			if (Speed > 50)
				PlaySound(ImpactSound, SLOT_Misc,0.65,,450,1.55);
			if (!Level.bDropDetail && Level.DetailMode != DM_Low && Level.TimeSeconds - LastSparkTime > 0.5 && EffectIsRelevant(Location,false))
			{
				Spawn(HitEffectClass,,, Location, Rotator(HitNormal));
				LastSparkTime = Level.TimeSeconds;
			}
		}
	}
}

function Explode(vector HitLocation, vector HitNormal)
{
	local Effect_ExplosionSpawner ExplosionSpawner;

	if (Role == ROLE_Authority)
	{
		ShakeView();
		MakeNoise(1.0);
	}
	ExplosionSpawner = spawn(ExplosionSpawnerClass, none,, Location, Rotation);
	ExplosionSpawner.DoExplosion(HitLocation, HitNormal);
	DelayedHurtRadius(Damage,DamageRadius, MyDamageType, MomentumTransfer, HitLocation);
	Destroy();
}

simulated function ProcessTouch(actor Other, vector HitLocation)
{
	if (Effect_BreakingGlass(Other) == None)
		return;

	// shatter glass
	Velocity = 0.85*Velocity;

	if (Role == ROLE_Authority)
		Other.TakeDamage(500, Instigator, HitLocation, Velocity, MyDamageType);
}


function TakeDamage(int Damage, Pawn instigatedBy, Vector HitLocation, Vector Momentum, class<DamageType> damageType)
{
	local vector start, end, hl, HitNormal, VelVector;
	local actor A;

	if (DamageType.Name != 'DamType_AKBayonet' && DamageType.Name != 'DamType_Machete' && damageType.Name != 'DamType_MachetePoison')
		DamageTaken += Damage;

	//Spawn(HitEffectClass,,, HitLocation, Rotator(-Momentum));

	// we want them to blow up if they take too much damage
	if (DamageTaken > MaxDamage)
	{
		// the person who destroyed it is now the explosion instigator
		if (InstigatedBy != None)
		{
			Instigator = InstigatedBy;
			if (InstigatedBy.Controller != None)
				InstigatorController = InstigatedBy.Controller;
		}

		Explode(location,vect(0,0,1));
		return;
	}
	else
	{
		// find out if we're on the ground, otherwise just apply normal momentum
		start = Location - vect(0,0,1)*CollisionHeight;
		end = start - vect(0,0,6);
		A = Trace(hl,HitNormal,end,start,false);
		if (hl != Vect(0,0,0))
		{
			SetPhysics(PHYS_Falling);
			// we want it to bounce away from the bullet hit, but we need to add lift first
			VelVector = 55*(-(HitNormal - (Momentum*17)));

			// reset bounce
			bBounce = true;

			if (Level.NetMode == NM_DedicatedServer)
				BoostVelocity = VelVector;
			Velocity += VelVector;
		}
		else
		{
			if (Level.NetMode == NM_DedicatedServer)
				BoostVelocity = 200*Momentum;
			Velocity += 200*Momentum;
		}
		RandSpin(50000);
	}
}

function ShakeView()
{
	local Controller C;
	local PlayerController PC;
	local float Dist, Scale;

	if (!bShakeView)
		return;

	for (C=Level.ControllerList;C!=None;C=C.NextController)
	{
		PC = PlayerController(C);
		if (PC != None && PC.ViewTarget != None)
		{
			Dist = VSize(Location - PC.ViewTarget.Location);
			if (Dist < DamageRadius * 2.0)
			{
				if (Dist < DamageRadius)
					Scale = 1.0;
				else
					Scale = FClamp((DamageRadius*2.0 - Dist) / (DamageRadius), 0, 1.0);
				FOPlayer(PC).ClientShake(vect(350,350,350)*Scale,vect(9000,9000,9000),3.5, vect(10,10,10)*Scale,vect(2000,2000,2000),3.5);
			}
		}
	}
}

defaultproperties
{
	// max allowed dmg
	MaxDamage=16

	// throw speeds
	Speed=1000
	MaxSpeed=3000

	// toss angle
	TossZ=+100.0

	// timer
	GrenExplodeTimer=2.8

	// added gravity
	GravityDampen=11.0

	// drawing
	DrawType=DT_StaticMesh
	DrawScale=0.35
	HitEffectClass=class'FragOps.Effect_KnifeSparkImpact'

	// splashes
	SplashSound=sound'FOGameSounds.AmbientHit.Kerplunk'
	SplashThreshold=700
	
	// settings
	RemoteRole=ROLE_SimulatedProxy
	Physics=PHYS_Falling
	bShakeView=true
	bBounce=true
	bCanBeDamaged=true
	bStatic=false
	bNoDelete=false
	bProjTarget=true
	bBlockZeroExtentTraces=true
	bBlockNonZeroExtentTraces=true
	bCollideActors=true
	bCollideWorld=true
	bWorldGeometry=false
	bNetInitialRotation=true
	bFixedRotationDir=true
	bReplicateInstigator=true
	bNetNotify=true
	bReplicateMovement=true
	bUpdateSimulatedPosition=true
	bNetTemporary=false
	AmbientGlow=90
	NetUpdateFrequency=140.0
	LifeSpan=10.0
	DesiredRotation=(Pitch=12000,Yaw=5666,Roll=2334)
	FluidSurfaceShootStrengthMod=3.f
	bUseCollisionStaticMesh=true

	// collision
	CollisionHeight=+5.0
	CollisionRadius=+5.0
}